/* usogres -- a replacement to postmaster tool distribute multiple postgres
 * Copyright (C) 2000-2002  HOSOKAWA Tetsuichi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: Socket.h,v 1.6 2001/04/18 07:14:43 hosokawa Exp $
 */

#ifdef WIN32
#include <winsock.h>
#include <string.h>
#include <fcntl.h>
#include <sys/types.h>
#else
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/select.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#endif

class SockAddr_in : public sockaddr_in {
public:
#ifdef WIN32
	SockAddr_in(int port, const char* sock_addr = (char *)INADDR_ANY, int family = AF_INET);
#else
	SockAddr_in(int port, const char* sock_addr = INADDR_ANY, int family = AF_INET);
#endif
};

class Socket {
#ifdef WIN32
public:
	int server_flg_org;
	int server_flg_copy;
#endif
	int sockfd;
	SockAddr_in sa;

public:
#ifdef WIN32
	Socket(int port, const char* sock_addr = (char *)INADDR_ANY,
#else
	Socket(int port, const char* sock_addr = INADDR_ANY,
#endif
			int type = SOCK_STREAM, int family = AF_INET, int protocol = 0);
	virtual ~Socket();

public:
	int is_valid();

public:
	int bind();
	int listen(int backlog = 5);
#ifdef WIN32
	int Accept();
#else
	int accept();
#endif
	int connect();
	int close();
	int set_sockfd(int fd) { return sockfd = fd; }
	int get_sockfd() const { return sockfd; }
	int set_option(int level, int optname);
	int set_non_blocking();
	int set_async();
	int set_send_buffer_size(int);
	int set_recv_buffer_size(int);

public:
	int readn(char*, int);
	int writen(const char*, int);
	int read(char* buf, int nbytes);
	int write(const char* buf, int nbytes);
};
