/* usogres -- a replacement to postmaster tool distribute multiple postgres
 * Copyright (C) 2000-2002  HOSOKAWA Tetsuichi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: SysLog.cc,v 1.2 2001/04/09 04:27:24 hosokawa Exp $
 */
#include "SysLog.h"

#ifndef WIN32

SysLog::SysLog(const char* ident, int option, int facility, int lvl)
{
	open(ident, option, facility, lvl);
}

void SysLog::open(const char* ident, int option, int facility, int lvl)
{
	openlog(ident, option, facility);
	set_level(lvl);
}

void SysLog::close()
{
	::closelog();
}

void SysLog::printf(const char* format ...)
{
	va_list ap;
	va_start(ap, format);

	char* pbuf = new char[strlen(format)*3];
	::vsprintf(pbuf, format, ap);
	printf(level, pbuf);
	delete [] pbuf;

	va_end(ap);
}

void SysLog::printf(int lvl, const char* format ...)
{
	va_list ap;
	va_start(ap, format);

	char* pbuf = new char[strlen(format)*3];
	::vsprintf(pbuf, format, ap);
	::syslog(lvl, pbuf);
	delete [] pbuf;

	va_end(ap);
}

#else	/* WIN32 */

SysLog::SysLog(char* ident)
{
	m_ident = ident;
	level = 0;
}

void SysLog::LogLevel(int loglevel)
{
	level = loglevel;
}

void SysLog::printf(char *fmt, ...)
{
	WORD EventType = EVENTLOG_AUDIT_SUCCESS;
	HANDLE  hEventSource;
	LPTSTR  lpszStrings[1];
	char Msg[200];
	va_list Marker;

	if( level > 0 )
	{
		va_start(Marker, fmt);
		_vsnprintf(Msg, sizeof Msg, fmt, Marker);
		puts(Msg);
		hEventSource = RegisterEventSource(NULL, m_ident);
		lpszStrings[0] = Msg;
		if (hEventSource != NULL) {
			ReportEvent(hEventSource, EventType, 0, 0, NULL, 1, 0, (const char **)lpszStrings, NULL);
			DeregisterEventSource(hEventSource);
		}
	}
}

void SysLog::printf(WORD EventType, char *fmt, ...)
{
	HANDLE  hEventSource;
	LPTSTR  lpszStrings[1];
	char Msg[200];
	va_list Marker;

	va_start(Marker, fmt);
	_vsnprintf(Msg, sizeof Msg, fmt, Marker);
	puts(Msg);
	hEventSource = RegisterEventSource(NULL,m_ident);

	lpszStrings[0] = Msg;

	if (hEventSource != NULL) {
		ReportEvent(hEventSource, EventType, 0, 0, NULL, 1, 0, (const char **)lpszStrings, NULL);
		DeregisterEventSource(hEventSource);
	}
}

#endif
