/* usogres -- a replacement to postmaster tool distribute multiple postgres
 * Copyright (C) 2000-2002  HOSOKAWA Tetsuichi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: exception.h,v 1.5 2001/06/29 01:32:51 hosokawa Exp $
 */
#ifndef _USOGRES_EXCEPTION_H_
#define _USOGRES_EXCEPTION_H_

#ifdef HAVE_CONFIG_H
#  undef PACKAGE
#  undef VERSION
#  include "../config.h"
#endif
#ifdef HAVE_STRING_H
#  include <string.h>
#endif

#include <exception>

class UsogresException : public std::exception {
	const char* fname;
	int lno;
	char* msg_buf;
public:
	UsogresException(const char* fn, int ln);
	virtual ~UsogresException();
	virtual const char* where();
};

class ImcompletedValueException : public UsogresException {
public:
	ImcompletedValueException(const char* fn, int ln) : UsogresException(fn, ln)  {}
	virtual const char* what() const { return "ImcompletedValueException"; }
};

class MemoryException : public UsogresException {
public:
  MemoryException(const char* fn, int ln) : UsogresException(fn, ln) {}
  virtual const char* what() const { return "MemoryException"; }
};

class FileException : public UsogresException {
public:
  FileException(const char* fn, int ln) : UsogresException(fn, ln) {}
  virtual const char* what() const { return "FileException"; }
};

class EnvException : public UsogresException {
public:
  EnvException(const char* fn, int ln) : UsogresException(fn, ln) {}
  virtual const char* what() const { return "EnvException"; }
};

class FileFormatException : public UsogresException {
public:
  FileFormatException(const char* fn, int ln) : UsogresException(fn, ln) {}
  virtual const char* what() const { return "FileFormatException"; }
};

class TokenFormatException : public UsogresException {
public:
  TokenFormatException(const char* fn, int ln) : UsogresException(fn, ln) {}
  virtual const char* what() const { return "TokenFormatException"; }
};

class LogicalException : public UsogresException {
public:
  LogicalException(const char* fn, int ln) : UsogresException(fn, ln) {}
  virtual const char* what() const { return "LogicalException"; }
};

class InvalidProtocolException : public UsogresException {
public:
  InvalidProtocolException(const char* fn, int ln) : UsogresException(fn, ln) {}
  virtual const char* what() const { return "InvalidProtocolException"; }
};

#endif
