/* usogres -- a replacement to postmaster tool distribute multiple postgres
 * Copyright (C) 2000-2002  HOSOKAWA Tetsuichi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: lock.cc,v 1.6 2001/05/14 17:03:49 hosokawa Exp $
 */

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#ifdef HAVE_ERROR_H
#  include <error.h>
#endif
#include <errno.h>

#include "SysLog.h"
#include "lock.h"

#if !defined(HAVE_STRERROR)
#  define  strerror(errnum)  sys_errlist[errnum]
#endif

extern SysLog sl_obj;

UsoLock::UsoLock()
{
	flock_struct.l_start = 0;
	flock_struct.l_whence = SEEK_SET;
	flock_struct.l_len = 1;
	flock_struct.l_pid = getpid();

	locksem = open("/tmp/.usogres.lock", O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);
	if (locksem == -1) {
		sl_obj.printf(LOG_ERR, "sem_open failed(%s).", strerror(errno));
	}
}

UsoLock::~UsoLock()
{
	if (locksem != -1) {
		close(locksem);
	} else {
		sl_obj.printf(LOG_ERR, "invalid lock.");
	}
}

void
UsoLock::lock()
{
	if (locksem != -1) {
		flock_struct.l_type = F_RDLCK;
		int ret = fcntl(locksem, F_SETLKW, &flock_struct);
		if (ret == -1) {
			sl_obj.printf(LOG_ERR, "fcntl failed(%s).", strerror(errno));
		}
	} else {
		sl_obj.printf(LOG_ERR, "invalid lock.");
	}
}

void
UsoLock::unlock()
{
	if (locksem != -1) {
		flock_struct.l_type = F_UNLCK;
		int ret = fcntl(locksem, F_SETLK, &flock_struct);
		if (ret == -1) {
			sl_obj.printf(LOG_ERR, "fcntl failed(%s).", strerror(errno));
		}
	} else {
		sl_obj.printf(LOG_ERR, "invalid lock.");
	}
}

char* UsoSem::path = "/tmp/.usofifo";
UsoSem::UsoSem()
{
	sem_fd[0] = sem_fd[1] = -1;
	sem_magic = 0;
	sl_obj.printf("UsoSem::UsoSem");
}

UsoSem::~UsoSem()
{
	sl_obj.printf("UsoSem::~UsoSem");
	if (sem_magic == SEM_MAGIC) {
		close();
	}
}

int
UsoSem::open(int oflag, ...)
{
	int flags, save_errno;
	char c;
	mode_t mode;
	va_list ap;
	unsigned int value = 0;

	if (oflag & O_CREAT) {
		va_start(ap, oflag);
		mode = va_arg(ap, mode_t);
		value = va_arg(ap, unsigned int);
		va_end(ap);
		if (mkfifo(path, mode) < 0) {
			if (errno == EEXIST && (oflag & O_EXCL)) {
				oflag &= ~O_CREAT;
			} else {
				return SEM_FAILED;
			}
		}
	}

	sem_fd[0] = sem_fd[1] = -1;
	if ((sem_fd[0] = ::open(path, O_RDONLY | O_NONBLOCK)) < 0) {
		goto error;
	}
	if ((sem_fd[1] = ::open(path, O_WRONLY | O_NONBLOCK)) < 0) {
		goto error;
	}

	if ((flags = ::fcntl(sem_fd[0], F_GETFL, 0)) < 0) {
		goto error;
	}
	flags &= ~O_NONBLOCK;
	if (::fcntl(sem_fd[0], F_SETFL, flags) < 0) {
		goto error;
	}
	if (oflag & O_CREAT) {
		for (unsigned int i = 0; i < value; i++) {
			if (::write(sem_fd[1], &c, 1) != 1) {
				goto error;
			}
		}
	}
	sem_magic = SEM_MAGIC;
	sl_obj.printf("UsoSem::open completed.");
	return 0;

error:
	save_errno = errno;
	if (oflag & O_CREAT) {
		::unlink(path);
	}
	::close(sem_fd[0]);
	::close(sem_fd[1]);
	errno = save_errno;
	sl_obj.printf(LOG_ERR, "UsoSem::open returns error.");
	return SEM_FAILED;
}

int
UsoSem::close()
{
	if (sem_magic != SEM_MAGIC) {
		errno = EINVAL;
		return -1;
	}
	sem_magic = 0;
	if (::close(sem_fd[0]) == -1 || ::close(sem_fd[1]) == -1) {
		return -1;
	}
	return 0;
}

int
UsoSem::unlink()
{
	return ::unlink(path);
}

int
UsoSem::post()
{
	char c;
	if (sem_magic != SEM_MAGIC) {
		errno = EINVAL;
		return -1;
	}
	if (write(sem_fd[1], &c, 1) == 1) {
		return 0;
	}
	return -1;
}

int
UsoSem::wait()
{
	char c;
	if (sem_magic != SEM_MAGIC) {
		errno = EINVAL;
		return -1;
	}

	if (read(sem_fd[0], &c, 1) == 1) {
		return 0;
	}
	return -1;
}
