/* usogres -- a replacement to postmaster tool distribute multiple postgres
 * Copyright (C) 2000-2002  HOSOKAWA Tetsuichi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: protocol.h,v 1.2 2001/04/09 04:27:25 hosokawa Exp $
 */
#ifndef _USOGRES_PROTOCOL_H_
#define _USOGRES_PROTOCOL_H_

#include <string>

#ifdef WIN32
typedef std::basic_string<char> ug_string;
#else
typedef basic_string<char> ug_string;
#endif

class ug_bytes;
class PgsqlProtocol {
	int length;
	int buf_length;
	char* buf;

public:
	enum Msg {
		Unknown,
		AsciiRow,
		AuthenticationOk,
		AuthenticationKerberosV4,
		AuthenticationKerberosV5,
		AuthenticationUnencryptedPassword,
		AuthenticationEncryptedPassword,
		BackendKeyData,
		BinaryRow,
		CancelRequest,
		CompletedResponse,
		CopyDataRows,
		CopyInResponse,
		CopyOutResponse,
		CursorResponse,
		EmptyQueryResponse,
		EncryptedPasswordPacket,
		ErrorResponse,
		FunctionCall,
		FunctionResultResponse,
		FunctionVoidResponse,
		NoticeResponse,
		NotificationResponse,
		Query,
		ReadyForQuery,
		RowDescription,
		StartupPacket,
		Terminate,
		UnencryptedPasswordPacket
	};

private:
	Msg type;

public:
	PgsqlProtocol(Msg protocol_type = Unknown);
	~PgsqlProtocol();

protected:
	int get_int(char* ptr, int size) const;
	char get_byte(char* ptr) const;
	const char* get_string(char* ptr) const;

protected:
	int get_length() const { return length; }
	const char* const get_buffer() const { return buf; }
	bool check_fixed_length_type(int len);
	bool check_id_only();
	bool check_id_and_string();
	bool check_AsciiRow();
	bool check_BinaryRow();
	bool check_CopyDataRows();
	bool check_EncryptedPasswordPacket();
	bool check_FunctionCall();
	bool check_ResultResponse();
	bool check_NotificationResponse();
	bool check_RowDescription();
	void dump_fixed_length_type(int len);
	void dump_id_only();
	void dump_id_and_string();
	void dump_AsciiRow();
	void dump_BinaryRow();
	void dump_CopyDataRows();
	void dump_EncryptedPasswordPacket();
	void dump_FunctionCall();
	void dump_ResultResponse();
	void dump_NotificationResponse();
	void dump_RowDescription();

public:
	bool add_byte(unsigned char ch);
	Msg get_protocol_type();
	bool is_completed();
	void dump();
};

void dump_protocol(char* buf, int size);

#endif
