/* usogres -- a replacement to postmaster tool distribute multiple postgres
 * Copyright (C) 2000-2002  HOSOKAWA Tetsuichi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: usogres.cc,v 1.22 2001/06/29 09:39:40 hosokawa Exp $
 */
#ifdef WIN32
#include "../config.h"
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#else
#ifdef HAVE_CONFIG_H
#  undef PACKAGE
#  undef VERSION
#  include "../config.h"
#endif

#include <signal.h>
#include <iostream>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/wait.h>
#endif

#ifdef HAVE_ERROR_H
#  include <error.h>
#endif

#include <errno.h>

#include "Socket.h"
#include "SysLog.h"
#include "debug.h"
#include "protocol.h"
#include "exception.h"
#include "rc.h"

#ifndef WIN32
#include "lock.h"
#endif
SysLog sl_obj("usogres");

static UsogresRC rc_obj;
const int sock_buf_size = 2048;
#ifndef WIN32
static UsoSem usosem;
#endif

#if !defined(HAVE_STRERROR)
#  define  strerror(errnum)  sys_errlist[errnum]
#endif

/*
static void dump_to_fp(FILE* fp, char b_or_f, const char* buf, int size)
{
	fprintf(fp, "%c", b_or_f);
	for (int i = 0; i < size; i++) {
		unsigned int c = (unsigned char)(buf[i]);
		fprintf(fp, " %02x", c);
	}
	fprintf(fp, "\n");
}

static int is_file_exist(const char* fn)
{
	if (fopen(fn, "r") == NULL && errno == ENOENT) {
		return false;
	}
	return true;
}
*/

static int get_max_fd(Socket& sock_in, Socket* p_org, Socket* p_copy)
{
	int max_fd = (((p_org) ? p_org->get_sockfd() : -1) > sock_in.get_sockfd())
					? p_org->get_sockfd() : sock_in.get_sockfd();
	max_fd = ((max_fd >= ((p_copy) ? p_copy->get_sockfd() : -1))
				? max_fd : p_copy->get_sockfd()) + 1;
	return max_fd;
}

static int is_sock_valid(Socket* p)
{
	sl_obj.printf("is_sock_valid(%p).", p);
	if (p == 0)
		return 0;

	int ret = p->is_valid();
	sl_obj.printf("is_sock_valid returns %d.", ret);
	return ret;
}

int backend_proc(Socket& sock_in)
{
	fd_set read_mask, read_mask_org, except_mask;
#ifdef WIN32
	sock_in.server_flg_org = sock_in.server_flg_copy = 0;
#else
	int server_flg_org = 0;
	int server_flg_copy = 0;
#endif

	sl_obj.printf("backend_proc started.");

	FD_ZERO(&read_mask);
	FD_SET(sock_in.get_sockfd(), &read_mask);

	Socket* sock_org = new Socket(rc_obj.get_portorg(), rc_obj.get_hostorg().c_str());
	if (sock_org != 0) {
		sl_obj.printf("new sock_org completed.");
		sock_org->set_option(IPPROTO_TCP, TCP_NODELAY);
		if (sock_org->connect() < 0) {
			sl_obj.printf(LOG_ERR, "sock_org->connect failed(%s)", strerror(errno));
			delete sock_org;
			sock_org = 0;
		} else {
			sl_obj.printf("sock_org->connect completed.");
			FD_SET(sock_org->get_sockfd(), &read_mask);
#ifdef WIN32
			sock_in.server_flg_org = 1;
#else
			server_flg_org = 1;
#endif
		}
	} else {
		sl_obj.printf(LOG_ERR, "sock_org is null!!");
	}

	Socket* sock_copy = 0;
#ifdef WIN32
	if (sock_in.server_flg_org) {
#else
	if (server_flg_org) {
#endif
		sock_copy = new Socket(rc_obj.get_portcopy(), rc_obj.get_hostcopy().c_str());
		if (sock_copy != 0) {
			sl_obj.printf("new sock_copy completed.");
			sock_copy->set_option(IPPROTO_TCP, TCP_NODELAY);
			if (sock_copy->connect() < 0) {
				sl_obj.printf(LOG_ERR, "sock_copy->connect failed(%s)", strerror(errno));
				delete sock_copy;
				sock_copy = 0;
			} else {
				sl_obj.printf("sock_copy->connect completed.");
				FD_SET(sock_copy->get_sockfd(), &read_mask);
#ifdef WIN32
				sock_in.server_flg_copy = 1;
#else
				server_flg_copy = 1;
#endif
			}
		} else {
			sl_obj.printf(LOG_ERR, "sock_copy is null!!");
		}
	} else {
		sock_org = new Socket(rc_obj.get_portcopy(), rc_obj.get_hostcopy().c_str());
		if (sock_org != 0) {
			sl_obj.printf(LOG_ERR, "retry new sock_copy completed.");
			sock_org->set_option(IPPROTO_TCP, TCP_NODELAY);
			if (sock_org->connect() < 0) {
				sl_obj.printf(LOG_ERR, "sock_org->connect failed again!!");
				delete sock_org;
				sock_org = 0;
			} else {
				sl_obj.printf(LOG_ERR, "retry new sock_copy connect completed.");
				FD_SET(sock_org->get_sockfd(), &read_mask);
#ifdef WIN32
				sock_in.server_flg_copy = 1;
#else
				server_flg_copy = 1;
#endif
			}
		} else {
			sl_obj.printf(LOG_ERR, "sock_org is null again!!");
		}
	}

#ifdef WIN32
	if (sock_in.server_flg_org + sock_in.server_flg_copy == 0) {
#else
	if (server_flg_org + server_flg_copy == 0) {
#endif
		sl_obj.printf(LOG_ERR, "sock_org and sock_copy cannot connect! exit!");
		return -1;
	}

	read_mask_org = read_mask;

	int max_fd = get_max_fd(sock_in, sock_org, sock_copy);
	sl_obj.printf("max_fd = %d", max_fd);
	while (1) {
		char sockbuf[sock_buf_size];
		int read_cnt = 0;

		int do_flag = 0;
		if (sock_in.is_valid()) {
			do_flag += (FD_ISSET(sock_in.get_sockfd(), &read_mask_org) ? 1 : 0) * 100;
		}
		if (is_sock_valid(sock_org)) {
			do_flag += (FD_ISSET(sock_org->get_sockfd(), &read_mask_org) ? 1 : 0) * 10;
		}
		if (is_sock_valid(sock_copy)) {
			do_flag += (FD_ISSET(sock_copy->get_sockfd(), &read_mask_org) ? 1 : 0) * 1;
		}
		if (do_flag <= 100) {
			sl_obj.printf("do_flag == %d, break!!", do_flag);
			break;
		}

		read_mask = read_mask_org;
		except_mask = read_mask_org;
		sl_obj.printf("selecting...%d", do_flag);
		int retval = ::select(max_fd, &read_mask, NULL, &except_mask, NULL);
		sl_obj.printf("select returns %d", retval);
		if (retval == -1) {
			sl_obj.printf("select returns the error!!");
			break;
		}

		sl_obj.printf("except_mask check start.");
		if (sock_in.is_valid()) {
			if (FD_ISSET(sock_in.get_sockfd(), &except_mask)) {
				sl_obj.printf("sock_in closed.");
				sock_in.close();
			}
		}
		if (is_sock_valid(sock_org)) {
			if (FD_ISSET(sock_org->get_sockfd(), &except_mask)) {
				sl_obj.printf("sock_org deleted.");
				delete sock_org;
				sock_org = 0;
			}
		}
		if (is_sock_valid(sock_copy)) {
			if (FD_ISSET(sock_copy->get_sockfd(), &except_mask)) {
				sl_obj.printf("sock_copy deleted.");
				delete sock_copy;
				sock_copy = 0;
			}
		}
		sl_obj.printf("all except_mask check completed.");

		if (sock_in.is_valid()) {
			if (FD_ISSET(sock_in.get_sockfd(), &read_mask)) {
				sl_obj.printf("sock_in is ready to read.");
				sl_obj.printf("enter critical section.");
				read_cnt = sock_in.read(sockbuf, sizeof(sockbuf));
				if (read_cnt > 0) {
					sl_obj.printf("read %d bytes.", read_cnt);
					if (is_sock_valid(sock_org)) {
						sock_org->writen(sockbuf, read_cnt);
					}
					if (is_sock_valid(sock_copy)) {
						sock_copy->writen(sockbuf, read_cnt);
					}
					sl_obj.printf("read from sock_in %d", read_cnt);
				} else if (read_cnt == -1) {
					sl_obj.printf(LOG_ERR, "sock_in.read failed(%s)", strerror(errno));
					FD_CLR(sock_in.get_sockfd(), &read_mask_org);
					sock_in.close();
				} else if (read_cnt == 0) {
					sl_obj.printf("sock_in reaches EOF.");
					FD_CLR(sock_in.get_sockfd(), &read_mask_org);
					sock_in.close();
				}
			}
		}
		sl_obj.printf("sock_in read_mask check completed.");

		if (is_sock_valid(sock_org)) {
			if (FD_ISSET(sock_org->get_sockfd(), &read_mask)) {
				sl_obj.printf("sock_org is ready to read.");
				sl_obj.printf("enter critical section.");
				read_cnt = sock_org->read(sockbuf, sizeof(sockbuf));
				if (read_cnt > 0) {
					sl_obj.printf("read %d bytes.", read_cnt);
					sock_in.writen(sockbuf, read_cnt);
					sl_obj.printf("read from sock_org %d", read_cnt);
				} else if (read_cnt == -1) {
					sl_obj.printf(LOG_ERR, "sock_org->read failed(%s)", strerror(errno));
					FD_CLR(sock_org->get_sockfd(), &read_mask_org);
					delete sock_org;
					sock_org = 0;
				} else if (read_cnt == 0) {
					sl_obj.printf("sock_org reaches EOF.");
					FD_CLR(sock_org->get_sockfd(), &read_mask_org);
					delete sock_org;
					sock_org = 0;
				}
				sl_obj.printf("leave critical section.");
			}
		}
		sl_obj.printf("sock_org read_mask check completed.");

		if (is_sock_valid(sock_copy)) {
			if (FD_ISSET(sock_copy->get_sockfd(), &read_mask)) {
				sl_obj.printf("sock_copy is ready to read.");
				sl_obj.printf("enter critical section.");
				read_cnt = sock_copy->read(sockbuf, sizeof(sockbuf));
				if (read_cnt > 0) {
					sl_obj.printf("read %d bytes.", read_cnt);
					sl_obj.printf("read from sock_copy %d", read_cnt);
				} else if (read_cnt == -1) {
					sl_obj.printf(LOG_ERR, "sock_copy->read failed(%s)", strerror(errno));
					FD_CLR(sock_copy->get_sockfd(), &read_mask_org);
					delete sock_copy;
					sock_copy = 0;
				} else if (read_cnt == 0) {
					sl_obj.printf("sock_copy reaches EOF.");
					FD_CLR(sock_copy->get_sockfd(), &read_mask_org);
					delete sock_copy;
					sock_copy = 0;
				}
				sl_obj.printf("leave critical section.");
			}
		}
		sl_obj.printf("sock_copy read_mask check completed.");
	}

	sl_obj.printf("backend_proc shutdown...");
	delete sock_org;
	delete sock_copy;
	sock_in.close();

	sl_obj.printf("backend_proc shutdown...completed.");
	return 0;
}

#ifdef __FAVOR_BSD
void sig_child(int pid)
{
	int pid;
	union wait status;

	while ((pid = wait3(&status, WNOHANG, (struct rusage *)0)) > 0)
		;
}
#endif

void daemon_start()
{
#ifndef WIN32
	int child_pid;

	if (getppid() != 1) {		// from inittab?
#ifdef SIGTTOU
		::signal(SIGTTOU, SIG_IGN);
#endif
#ifdef SIGTTIN
		::signal(SIGTTIN, SIG_IGN);
#endif
#ifdef SIGTSTP
		::signal(SIGTSTP, SIG_IGN);
#endif

		if ((child_pid = fork()) < 0) {
			sl_obj.printf(LOG_ERR, "fork failed.");
			exit(0);
		} else if (child_pid > 0) {
			sl_obj.printf("foreground job completed.");
			exit(0);
		}

		if (setpgid(getpid(), getpgrp()) == -1) {
			sl_obj.printf(LOG_ERR, "can't change process group.");
			exit(0);
		}
		signal(SIGHUP, SIG_IGN);
		if ((child_pid = fork()) < 0) {
			sl_obj.printf(LOG_ERR, "can't fork second child.");
			exit(0);
		} else if (child_pid > 0) {
			sl_obj.printf("first step child process completed.");
			exit(0);
		}
	}

	// close all file descriptor
	for (int i = 0; i < getdtablesize(); i++) {
		::close(i);
	}

	// change directory to root
	chdir("/");

	// clear inherited file mode creation mask
	umask(0);

	// for child process
#ifdef __FAVOR_BSD
	::signal(SIGCHLD, sig_child);
#else
	::signal(SIGCHLD, SIG_IGN);
#endif

#endif	/* WIN32 */
}

#ifdef WIN32
/* Windows Thread */
VOID backend_thread(PVOID Parameter)
{
	Socket *sock_prm = (Socket *)Parameter;
	Socket *sock_th = new Socket(rc_obj.get_port(), rc_obj.get_host().c_str());
	sock_th->sockfd = sock_prm->sockfd;
	memset(&sock_th->sa, 0, sizeof(sock_th->sa));
	sock_th->sa.sin_addr = sock_prm->sa.sin_addr;
	sock_th->sa.sin_family = sock_prm->sa.sin_family;
	sock_th->sa.sin_port = sock_prm->sa.sin_port;
	backend_proc(*sock_th);
	delete sock_th;
}
#endif

int main(int argc, char* argv[])
{
#ifdef WIN32
	int Err;
	WSADATA WsaData;
	SECURITY_ATTRIBUTES SecurityAttributes;
	HANDLE ThreadHandle;
	DWORD ThreadId;

	sl_obj.LogLevel(rc_obj.get_loglevel());

	if ((Err = WSAStartup(MAKEWORD(1, 1), &WsaData)) != 0) {
		sl_obj.printf("WSAStartup: %d\n", Err);
		return 0;
	}
#endif
	sl_obj.printf("start");
#ifndef WIN32
	daemon_start();
	usosem.unlink();
	usosem.open(O_CREAT, S_IRWXU, rc_obj.get_max_process());
#endif

	Socket sock_serv(rc_obj.get_port(), rc_obj.get_host().c_str());
	sock_serv.set_option(SOL_SOCKET, SO_REUSEADDR);

	sl_obj.printf("sock_serv(%d, %s) : ok", rc_obj.get_port(), rc_obj.get_host().c_str());
	if (sock_serv.bind() < 0) {
		sl_obj.printf(LOG_ERR, "sock_serv.bind() failed(%s)", strerror(errno));
		return -1;
	}
	sl_obj.printf("sock_serv.bind() : ok");
	if (sock_serv.listen() < 0) {
		sl_obj.printf(LOG_ERR, "sock_serv.listen() failed(%s)", strerror(errno));
		return -1;
	}
	sl_obj.printf("sock_serv.listen() : ok");
	Socket new_sock(0, rc_obj.get_host().c_str());
#ifndef WIN32
	UsoSem wait_obj;
	wait_obj.open();
#endif
	while (1) {
		sl_obj.printf("waiting client connection...");
#ifdef WIN32
		new_sock.set_sockfd(sock_serv.Accept());
#else
		new_sock.set_sockfd(sock_serv.accept());
#endif
		if (new_sock.get_sockfd() < 0) {
			sl_obj.printf(LOG_ERR, "sock_serv.accept() failed(%s)", strerror(errno));
			return -1;
		}
		sl_obj.printf("sock_serv.accept() : ok");
		new_sock.set_option(IPPROTO_TCP, TCP_NODELAY);

#ifdef WIN32
		pid_t pid = getpid();
#else
		wait_obj.wait();
		pid_t pid = fork();
#endif
		if (pid < 0) {
			sl_obj.printf(LOG_ERR, "fork() failed(%s)", strerror(errno));
			sleep(1);
#ifndef WIN32
		} else if (pid == 0) {
			sock_serv.close();
#else
		} else {
#endif
			sl_obj.printf("CHECK-START::backend_proc(%d)", getpid());
#ifdef WIN32
			SecurityAttributes.nLength = sizeof(SecurityAttributes);
			SecurityAttributes.lpSecurityDescriptor = NULL; 
			SecurityAttributes.bInheritHandle = FALSE; 
			ThreadHandle = 
            		CreateThread(&SecurityAttributes, 0, 
                         (LPTHREAD_START_ROUTINE) backend_thread,
                         (LPVOID)&new_sock, 0, &ThreadId);
			if (ThreadHandle == NULL) {
				sl_obj.printf("CreateThread:error(%d)\n", GetLastError());
				continue;
			}
			CloseHandle(ThreadHandle);
#else
			backend_proc(new_sock);
#endif
			sl_obj.printf("CHECK-E N D::backend_proc(%d)", getpid());
#ifndef WIN32
			UsoSem post_obj;
			post_obj.open();
			post_obj.post();
			exit(0);
		} else {
			sl_obj.printf("fork() : ok");
			new_sock.close();
#endif
		}
	}

	return 0;
}
